/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

int nobootsplash = 0;

int
main (int argc, char *argv[])
{
  register int i;
  int mainwindow = 0;
  char *server_module = NULL;

  for (i = 1; i != argc; i++)
    {
      if (!strcmp (argv[i], "-s") || !strcmp (argv[i], "--server"))
	{
	  if (!argv[i + 1])
	    fatal ("Error: %s need a argument!", argv[i]);

	  i++;
	  server_module = argv[i];
	}

      else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--time"))
	{
	  if (!argv[i + 1])
	    fatal ("Error: %s need a argument!", argv[i]);

	  i++;
	  tooltip_time = atoi (argv[i]);
	}

      else if (!strcmp (argv[i], "-nb")
	       || !strcmp (argv[i], "--nobootsplash"))
	{
	  nobootsplash = 1;
	}

      else if (!strcmp (argv[i], "-c") || !strcmp (argv[i], "--color"))
	{
	  if (!argv[i + 1])
	    fatal ("Error: %s need a argument!", argv[i]);

	  i++;
	  graphic_color = strdup (argv[i]);
	}

      else if (!strcmp (argv[i], "-m") || !strcmp (argv[i], "--mainwindow"))
	{
	  mainwindow = 1;
	}

      else if (!strcmp (argv[i], "-i") || !strcmp (argv[i], "--image"))
	{
	  if (!argv[i + 1])
	    fatal ("Error: %s need a argument!", argv[i]);

	  i++;
	  graphic_image = strdup (argv[i]);
	}

      else if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help"))
	{
	  usage (0);
	}
      else
	usage (1);
    }

  gtk_init (NULL, NULL);

  modules_xml_delete ();

  if (modules_readdir (server_module))
    {
      fatal ("Error: No modules directory.");
    }
  else if (!module_head)
    {
      fatal ("Error: No modules loaded.");
    }

  tooltips_check_id ();

  config_read ();

  if (!mainwindow)
    {
      graphic_init ();

      if (tooltip_head)
	tooltips_timer ();

      tooltips_timeout (NULL);
    }
  else
    {
      tutor_init ();
      tutor_container (tutor_container_default, NULL);
    }

  if (server_module)
    modules_xml_delete ();

  gtk_main ();

  tooltips_close ();
  modules_close ();
  config_write ();

  return 0;
}

void
fatal (char *str, ...)
{
  va_list va;
  va_start (va, str);

  fprintf (stderr, "* ");
  vfprintf (stderr, str, va);
  fprintf (stderr, "\n");

  va_end (va);

  exit (1);
}

void
msg (char *str, ...)
{
  va_list va;
  va_start (va, str);

  fprintf (stderr, "* ");
  vfprintf (stderr, str, va);
  fprintf (stderr, "\n");

  va_end (va);
}

void
usage (int r)
{
  fprintf (stderr, "%s %s - %s - %s\n\n", PACKAGE, VERSION, AUTHOR,
	   COPYRIGHT);
  fprintf (stderr,
	   "\t-s|--server           <remote_file> remote file list of xml modules.\n");
  fprintf (stderr,
	   "\t-t|--time             <number> minutes between a tip and the other.\n");
  fprintf (stderr, "\t-nb|--nobootsplash    no boot splash\n");
  fprintf (stderr,
	   "\t-i|--image            <image> use this image like logo.\n");
  fprintf (stderr, "\t-h|--help             this help.\n");

  fprintf (stderr, "\n");

  exit (r);
}

char *
trim (char *tmp)
{
  int i = 0;
  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    tmp++;

  i = strlen (tmp);
  i--;

  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    i--;

  tmp[i + 1] = 0;

  return tmp;
}

/* EOF */
